// SOAPElement.h
//
// Kenn Scribner
// Understanding SOAP, March 2000
//
// Basic SOAP XML element object definition.
// Implements the basic SOAP (XML) node and acts
// as the base class for all derived SOAP elements
// (envelope, header, body, independent, and
// embedded).
//
#if !defined __SOAPELEMENT_H
#define __SOAPELEMENT_H

#include <string>
#include <iostream>
#include <strstream>
#include "SOAPElementAttributes.h"
#include "SOAPElementNamespace.h"

// SOAP error codes
#define SOAP_S_OK			0x00000000
#define SOAP_S_FALSE		0x00000001
#define SOAP_E_POINTER		0x80004003
#define SOAP_E_OUTOFMEMORY	0x8007000E
#define SOAP_E_INVALIDARG	0x80070057
#define SOAP_E_UNEXPECTED	0x8000ffff

class CSOAPElement
{
public:
	CSOAPElement(CSOAPElement* pParent = NULL);
	virtual ~CSOAPElement();

// Methods
public:
	void SetTagText(const std::string& strTagText);
	void SetTagNamespace(long iNamespaceCookie);
	bool EnableStdAttribute(long iStdAttributeCookie, bool bEnable);
	unsigned long InsertAttribute(long iNamespaceCookie, std::string& strAttrName, std::string& strAttrValue);
	void SetElementData(const std::string& strData, bool bIsCDATA = false);
	bool EnableStdNamespace(long iStdNamespaceCookie, bool bEnable);
	unsigned long InsertNamespace(const std::string& strLongName, const std::string& strShortName, long* piNamespaceCookie);
	unsigned long GetNamespace(long iNamespaceCookie, std::string& strNamespace);
	virtual unsigned long WriteToStream(std::ostrstream& ostr);
	virtual unsigned long StreamContents(std::ostrstream& ostr);
	virtual unsigned long StreamOpenTag(std::ostrstream& ostr);
	virtual unsigned long StreamCloseTag(std::ostrstream& ostr);

	CSOAPElement* GetParent() { return m_pParent; }

// Attributes
protected:
	CSOAPElement* m_pParent;
	CSOAPElementAttributes m_CAttributes;
	CSOAPElementNamespace m_CNamespaces;

	long m_iTagNamespaceCookie;

	std::string m_strTagText;
	std::string m_strData;

};

#endif // __SOAPELEMENT_H