// SOAPElementNamespace.h
//
// Kenn Scribner
// Understanding SOAP, March 2000
//
// SOAP XML element namespace object definition.
//
#if !defined __SOAPELEMENTNAMESPACE_H
#define __SOAPELEMENTNAMESPACE_H

#include <vector>
#include <string>
#include <map>
#include <iostream>
#include <strstream>

// Standard namespace cookies
#define NULLNSCOOKIE	-1
#define ENVNSCOOKIE		0
#define ENCNSCOOKIE		1
#define XSINSCOOKIE		2
#define COMNSCOOKIE		3

// Fwd declaration
class CSOAPElement;

class CSOAPElementNamespace
{
public:
	CSOAPElementNamespace(CSOAPElement* pParent = NULL, long iBaseNSCookie = NULLNSCOOKIE);
	virtual ~CSOAPElementNamespace();

// Methods
public:
	bool EnableStdNamespace(long iNamespaceCookie, bool bEnable);
	unsigned long InsertNamespace(const std::string& strLongName, const std::string& strShortName, long* piNamespaceCookie);
	unsigned long GetNamespace(long iNamespaceCookie, std::string& strNamespace);
	unsigned long WriteToStream(std::ostrstream& ostr);

	void SetBaseNSCookie(long iNamespaceCookie = NULLNSCOOKIE)
	{
		m_iBaseNSCookie = iNamespaceCookie;
	}
	void SetParent(CSOAPElement* pNewParent) { m_pParent = pNewParent; }
	CSOAPElement* GetParent() { return m_pParent; }

// Attributes
protected:
	CSOAPElement* m_pParent;

	long m_iBaseNSCookie;
	long m_iNextCookie;

	bool m_bStdNamespaces[4];
	std::map<long,std::string> m_mapNSFull;
	std::map<long,std::string> m_mapNSShort;

};

#endif // __SOAPELEMENTNAMESPACE_H