// SOAPHttpPacket.h
//
// Kenn Scribner
// Understanding SOAP, March 2000
//
// SOAP HTTP object definition
//
#if !defined __SOAPHTTPPACKET_H
#define __SOAPHTTPPACKET_H

#include <string>
#include <iostream>
#include <strstream>
#include "SOAPEnvelope.h"

typedef struct tagSOAPCallInfo {
	std::string strHost;
	std::string strPostHdr;
	std::string strActionData;
	std::string strMethod;
} SOAPCALLINFO, *LPSOAPCALLINFO;

class CSOAPHttpPacket
{
public:
	CSOAPHttpPacket(LPSOAPCALLINFO pci);
	virtual ~CSOAPHttpPacket();

// Methods
public:
	unsigned long WriteToStream(std::ostrstream& ostr, bool bEnvelopeOnly = false);
	CSOAPEnvelope* GetEnvelope() { return m_pEnvelope; }
	void GetMethodName(std::string& str)
	{
		str = m_strMethodName;
	}
	void GetEndpointURL(std::string& str)
	{
		str = m_strEndpointURL;
	}
	void GetPostText(std::string& str)
	{
		str = m_strPostText;
	}
	void GetPostHdr(std::string& str)
	{
		str = m_strPostHdr;
	}
	void GetHostText(std::string& str)
	{
		str = m_strHostText;
	}
	void GetHostHdr(std::string& str)
	{
		str = m_strHostHdr;
	}
	void GetContentTypeText(std::string& str)
	{
		str = m_strContentTypeText;
	}
	void GetContentTypeHdr(std::string& str)
	{
		str = m_strContentTypeHdr;
	}
	void GetContentLenText(std::string& str)
	{
		str = m_strContentLenText;
	}
	void GetContentLenHdr(std::string& str)
	{
		str = m_strContentLenHdr;
	}
	void GetSoapActionText(std::string& str)
	{
		str = m_strSOAPActionText;
	}
	void GetSoapActionHdr(std::string& str)
	{
		str = m_strSOAPActionHdr;
	}

// Attributes
protected:
	CSOAPEnvelope* m_pEnvelope;

	// Incoming HTTP informational strings
	std::string m_strEndpointURL;
	std::string m_strPostData;
	std::string m_strActionData;
	std::string m_strMethodName;

	// HTTP header strings
	int m_iContentLength;
	std::string m_strPostHdr; // string in header form
	std::string m_strPostText; // header data only
	std::string m_strHostHdr;
	std::string m_strHostText;
	std::string m_strContentTypeHdr;
	std::string m_strContentTypeText;
	std::string m_strContentLenHdr;
	std::string m_strContentLenText;
	std::string m_strSOAPActionHdr;
	std::string m_strSOAPActionText;

};

#endif // __SOAPHTTPPACKET_H