// VCDSoapExtenstion.cpp : implementation file
//

#include "stdafx.h"
#include "VCDSoapExtension.h"
#include "Activator.h"
#include <msxml.h>
#include <string>

CVCDSoapExtension theExtension;

// The method header field buffer size
const int HEADER_BUF_SIZE = 1024;

// Returns the version of the ISAPI extension to the IIS server
BOOL CVCDSoapExtension::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
    pVer->dwExtensionVersion = MAKELONG(HSE_VERSION_MINOR, HSE_VERSION_MAJOR);
    _tcscpy(pVer->lpszExtensionDesc, _T("SOAPExtension"));
    return TRUE;
}

// This method is triggered upon HTTP events within IIS
DWORD CVCDSoapExtension::HttpExtensionProc(EXTENSION_CONTROL_BLOCK *lpECB)
{
	// Make sure this was a POST request
	std::string strHTTPVerb(lpECB->lpszMethod);
	if ( strHTTPVerb != std::string("POST") ) {
		LPTSTR strMsg = _T("You found it...Kenn's SOAP demo ISAPI DLL!");
		ULONG bufLen = _tcslen(strMsg);
		lpECB->WriteClient(lpECB->ConnID, strMsg, &bufLen, HSE_IO_SYNC);     
		return HSE_STATUS_SUCCESS;
	} // if

	// Initialize COM runtime
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if ( FAILED(hr) ) return HSE_STATUS_ERROR;

	// Try to process the request
	DWORD dwRetCode = HSE_STATUS_SUCCESS;
	try {
		// Create the request DOM
        CComPtr<IXMLDOMDocument> pRequest;
        HRESULT hr = pRequest.CoCreateInstance(__uuidof(DOMDocument));
        if ( FAILED(hr) ) throw "Client.InitializationError#Unable to initialize the XML parser";

		// Load POSTed XML into DOM
        CComBSTR bstrTemp((char *)lpECB->lpbData);
  		VARIANT_BOOL bSuccess = false;
        pRequest->loadXML(bstrTemp,&bSuccess);
        if ( FAILED(hr) ) throw "Client.RequestError#Unable to load SOAP data into the XML parser";
        if ( !bSuccess ) throw "Client.RequestError#Unable to load SOAP data into the XML parser";

		// Retrieve the "SOAPAction:" HTTP header field
		DWORD dwMethodHeaderLen = HEADER_BUF_SIZE;
		TCHAR szMethodHeader[HEADER_BUF_SIZE];
		BOOL bRC = lpECB->GetServerVariable(lpECB->ConnID, 
											"HTTP_SOAPACTION",
											szMethodHeader, 
											&dwMethodHeaderLen);
		if ( bRC == FALSE ) throw "Client.InvalidSOAPPacket#Missing a valid SOAPAction header";

		// Find the delimeter between URI and method name
		LPTSTR szMethodURI = _tcstok(szMethodHeader,"#");
		LPTSTR szMethod = _tcstok(NULL,"\0");

		// Create an Activator for this method
        CComBSTR bstrMethodURI(szMethodURI);
        CComBSTR bstrMethod(szMethod);
        CActivator act(bstrMethodURI,bstrMethod);

		// Invoke the object's method and retrieve a response
        CComPtr<IXMLDOMDocument> pResponse;
        act.Invoke(pRequest,&pResponse);

		// Retrieve the XML response
		if ( !pResponse.p ) throw "Client.ResponseError#Unable to retrieve XML/SOAP response from the XML parser";
        CComBSTR bstrXML;
		hr = pResponse->get_xml(&bstrXML);
		if ( FAILED(hr) ) throw "Client.ResponseError#Unable to retrieve XML/SOAP response from the XML parser";

		// Check the length of the result...if it is an empty
		// string, something horrid happened and we need to
		// return a (hardcoded) SOAP fault from here.
		if ( !bstrXML.Length() ) throw "Client.ResponseError#Unable to retrieve XML/SOAP response from the XML parser";

		// Send the response to the client
		ULONG bufLen = bstrXML.Length();
		USES_CONVERSION;
		lpECB->WriteClient(lpECB->ConnID, (void*)W2A(bstrXML), &bufLen, HSE_IO_SYNC);     
    } // try
	catch(LPCTSTR strErr) {
		// The error string consists of two parts...the faultcode and
		// faultstring, which are separated by '#'.  Separate those
		// parts for later formatting into a fault packet.
		dwRetCode = HSE_STATUS_ERROR;
		TCHAR strMsg[1024] = {0};
		_tcscpy(strMsg,strErr); // incoming string is const...
		LPTSTR szFaultCode = _tcstok(strMsg,"#");
		LPTSTR szFaultString = _tcstok(NULL,"\0");

		// Create a string with the formatted fault packet
		std::string strFormat = "<SOAP-ENV:Envelope";
		strFormat += "xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/' ";
		strFormat += "SOAP-ENV:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>";
		strFormat += "<SOAP-ENV:Body><SOAP-ENV:Fault<faultcode>>";
		strFormat += szFaultCode;
		strFormat += "</faultcode><faultstring>";
		strFormat += szFaultString;
		strFormat += "</faultstring></SOAP-ENV:Fault></SOAP-ENV:Body></SOAP-ENV:Envelope>";

		// Return the fault packet
		ULONG bufLen = strFormat.length();
		lpECB->WriteClient(lpECB->ConnID, (void*)strFormat.c_str(), &bufLen, HSE_IO_SYNC);     
	} // catch
    catch(...) {
		// Unknown error, so return SOAP fault...  We'll assume the
		// problem was with the POST and not the GET...
		dwRetCode = HSE_STATUS_ERROR;
		LPTSTR strMsg = "<SOAP-ENV:Envelope"
                        "xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/' "
						"SOAP-ENV:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>"
					    "<SOAP-ENV:Body>"
						"<SOAP-ENV:Fault>"
						"<faultcode>Client.UnknownError</faultcode>"
						"<faultstring>Call your broker and dump all EnduraSoft stock immediately</faultstring>"
						"</SOAP-ENV:Fault>"
						"</SOAP-ENV:Body>"
						"</SOAP-ENV:Envelope>";
		ULONG bufLen = _tcslen(strMsg);
		lpECB->WriteClient(lpECB->ConnID, strMsg, &bufLen, HSE_IO_SYNC);     
	} // catch

	CoUninitialize();

	return dwRetCode;
}

